@@ -35,6 +35,7 @@ enum SourmashErrorCode {
   SOURMASH_ERROR_CODE_INVALID_HASH_FUNCTION = 1104,
   SOURMASH_ERROR_CODE_READ_DATA = 1201,
   SOURMASH_ERROR_CODE_STORAGE = 1202,
+  SOURMASH_ERROR_CODE_HLL_PRECISION_BOUNDS = 1301,
   SOURMASH_ERROR_CODE_IO = 100001,
   SOURMASH_ERROR_CODE_UTF8_ERROR = 100002,
   SOURMASH_ERROR_CODE_PARSE_INT = 100003,
@@ -45,6 +46,8 @@ typedef uint32_t SourmashErrorCode;
 
 typedef struct SourmashComputeParameters SourmashComputeParameters;
 
+typedef struct SourmashHyperLogLog SourmashHyperLogLog;
+
 typedef struct SourmashKmerMinHash SourmashKmerMinHash;
 
 typedef struct SourmashNodegraph SourmashNodegraph;
@@ -115,6 +118,40 @@ bool computeparams_track_abundance(const SourmashComputeParameters *ptr);
 
 uint64_t hash_murmur(const char *kmer, uint64_t seed);
 
+void hll_add_hash(SourmashHyperLogLog *ptr, uint64_t hash);
+
+void hll_add_sequence(SourmashHyperLogLog *ptr, const char *sequence, uintptr_t insize, bool force);
+
+uintptr_t hll_cardinality(const SourmashHyperLogLog *ptr);
+
+double hll_containment(const SourmashHyperLogLog *ptr, const SourmashHyperLogLog *optr);
+
+void hll_free(SourmashHyperLogLog *ptr);
+
+SourmashHyperLogLog *hll_from_buffer(const char *ptr, uintptr_t insize);
+
+SourmashHyperLogLog *hll_from_path(const char *filename);
+
+uintptr_t hll_intersection_size(const SourmashHyperLogLog *ptr, const SourmashHyperLogLog *optr);
+
+uintptr_t hll_ksize(const SourmashHyperLogLog *ptr);
+
+uintptr_t hll_matches(const SourmashHyperLogLog *ptr, const SourmashKmerMinHash *mh_ptr);
+
+void hll_merge(SourmashHyperLogLog *ptr, const SourmashHyperLogLog *optr);
+
+SourmashHyperLogLog *hll_new(void);
+
+void hll_save(const SourmashHyperLogLog *ptr, const char *filename);
+
+double hll_similarity(const SourmashHyperLogLog *ptr, const SourmashHyperLogLog *optr);
+
+const uint8_t *hll_to_buffer(const SourmashHyperLogLog *ptr, uintptr_t *size);
+
+void hll_update_mh(SourmashHyperLogLog *ptr, const SourmashKmerMinHash *optr);
+
+SourmashHyperLogLog *hll_with_error_rate(double error_rate, uintptr_t ksize);
+
 void kmerminhash_add_from(SourmashKmerMinHash *ptr, const SourmashKmerMinHash *other);
 
 void kmerminhash_add_hash(SourmashKmerMinHash *ptr, uint64_t h);
@@ -158,7 +195,12 @@ void kmerminhash_hash_function_set(SourmashKmerMinHash *ptr, HashFunctions hash_
 
 bool kmerminhash_hp(const SourmashKmerMinHash *ptr);
 
-uint64_t kmerminhash_intersection(const SourmashKmerMinHash *ptr, const SourmashKmerMinHash *other);
+SourmashKmerMinHash *kmerminhash_intersection(const SourmashKmerMinHash *ptr,
+                                              const SourmashKmerMinHash *other);
+
+uint64_t kmerminhash_intersection_union_size(const SourmashKmerMinHash *ptr,
+                                             const SourmashKmerMinHash *other,
+                                             uint64_t *union_size);
 
 bool kmerminhash_is_compatible(const SourmashKmerMinHash *ptr, const SourmashKmerMinHash *other);
 
@@ -174,17 +216,17 @@ SourmashStr kmerminhash_md5sum(const SourmashKmerMinHash *ptr);
 
 void kmerminhash_merge(SourmashKmerMinHash *ptr, const SourmashKmerMinHash *other);
 
-SourmashKmerMinHash *kmerminhash_new(uint32_t n,
+SourmashKmerMinHash *kmerminhash_new(uint64_t scaled,
                                      uint32_t k,
-                                     bool prot,
-                                     bool dayhoff,
-                                     bool hp,
+                                     HashFunctions hash_function,
                                      uint64_t seed,
-                                     uint64_t mx,
-                                     bool track_abundance);
+                                     bool track_abundance,
+                                     uint32_t n);
 
 uint32_t kmerminhash_num(const SourmashKmerMinHash *ptr);
 
+void kmerminhash_remove_from(SourmashKmerMinHash *ptr, const SourmashKmerMinHash *other);
+
 void kmerminhash_remove_hash(SourmashKmerMinHash *ptr, uint64_t h);
 
 void kmerminhash_remove_many(SourmashKmerMinHash *ptr,
@@ -193,6 +235,14 @@ void kmerminhash_remove_many(SourmashKmerMinHash *ptr,
 
 uint64_t kmerminhash_seed(const SourmashKmerMinHash *ptr);
 
+const uint64_t *kmerminhash_seq_to_hashes(SourmashKmerMinHash *ptr,
+                                          const char *sequence,
+                                          uintptr_t insize,
+                                          bool force,
+                                          bool bad_kmers_as_zeroes,
+                                          bool is_protein,
+                                          uintptr_t *size);
+
 void kmerminhash_set_abundances(SourmashKmerMinHash *ptr,
                                 const uint64_t *hashes_ptr,
                                 const uint64_t *abunds_ptr,
