@@ -1,7 +1,8 @@
 use finch::sketch_schemes::mash::MashSketcher;
 use finch::sketch_schemes::SketchScheme;
 
-use crate::sketch::minhash::{HashFunctions, KmerMinHash};
+use crate::encodings::HashFunctions;
+use crate::sketch::minhash::KmerMinHash;
 
 /*
  TODO:
@@ -14,12 +15,12 @@ impl From<MashSketcher> for KmerMinHash {
         let values = other.to_vec();
 
         let mut new_mh = KmerMinHash::new(
-            values.len() as u32,
+            0,
             values.get(0).unwrap().kmer.len() as u32,
             HashFunctions::murmur64_DNA,
             42,
-            0,
             true,
+            values.len() as u32,
         );
 
         let hash_with_abunds: Vec<(u64, u64)> = values
@@ -39,10 +40,10 @@ impl From<MashSketcher> for KmerMinHash {
 mod test {
     use std::collections::HashMap;
     use std::collections::HashSet;
-    use std::iter::FromIterator;
 
+    use crate::encodings::HashFunctions;
     use crate::signature::SigsTrait;
-    use crate::sketch::minhash::{HashFunctions, KmerMinHash};
+    use crate::sketch::minhash::KmerMinHash;
 
     use finch::sketch_schemes::mash::MashSketcher;
     use needletail::kmer::CanonicalKmers;
@@ -52,7 +53,7 @@ mod test {
 
     #[test]
     fn finch_behavior() {
-        let mut a = KmerMinHash::new(20, 10, HashFunctions::murmur64_DNA, 42, 0, true);
+        let mut a = KmerMinHash::new(0, 10, HashFunctions::murmur64_DNA, 42, true, 20);
         let mut b = MashSketcher::new(20, 10, 42);
 
         let seq = b"TGCCGCCCAGCACCGGGTGACTAGGTTGAGCCATGATTAACCTGCAATGA";
@@ -66,8 +67,8 @@ mod test {
 
         let b_hashes = b.to_vec();
 
-        let s1: HashSet<_> = HashSet::from_iter(a.mins().into_iter());
-        let s2: HashSet<_> = HashSet::from_iter(b_hashes.iter().map(|x| x.hash as u64));
+        let s1: HashSet<_> = a.mins().into_iter().collect();
+        let s2: HashSet<_> = b_hashes.iter().map(|x| x.hash as u64).collect();
         let i1 = &s1 & &s2;
 
         assert!(i1.len() == a.size());
@@ -75,7 +76,7 @@ mod test {
 
         if let Some(abunds) = a.abunds() {
             let mins = a.mins();
-            let smap: HashMap<_, _> = HashMap::from_iter(mins.iter().zip(abunds.iter()));
+            let smap: HashMap<_, _> = mins.iter().zip(abunds.iter()).collect();
             println!("{:?}", smap);
             for item in b_hashes.iter() {
                 assert!(smap.contains_key(&(item.hash as u64)));
@@ -89,7 +90,7 @@ mod test {
 
     #[test]
     fn from_finch() {
-        let mut a = KmerMinHash::new(20, 10, HashFunctions::murmur64_DNA, 42, 0, true);
+        let mut a = KmerMinHash::new(0, 10, HashFunctions::murmur64_DNA, 42, true, 20);
         let mut b = MashSketcher::new(20, 10, 42);
 
         let seq = b"TGCCGCCCAGCACCGGGTGACTAGGTTGAGCCATGATTAACCTGCAATGA";
@@ -103,8 +104,8 @@ mod test {
 
         let c = KmerMinHash::from(b);
 
-        let s1: HashSet<_> = HashSet::from_iter(a.mins().into_iter());
-        let s2: HashSet<_> = HashSet::from_iter(c.mins().into_iter());
+        let s1: HashSet<_> = a.mins().into_iter().collect();
+        let s2: HashSet<_> = c.mins().into_iter().collect();
         let i1 = &s1 & &s2;
 
         assert!(i1.len() == a.mins().len());
@@ -113,9 +114,9 @@ mod test {
         if let Some(a_abunds) = a.abunds() {
             if let Some(c_abunds) = c.abunds() {
                 let a_mins = a.mins();
-                let a_smap: HashMap<_, _> = HashMap::from_iter(a_mins.iter().zip(a_abunds.iter()));
+                let a_smap: HashMap<_, _> = a_mins.iter().zip(a_abunds.iter()).collect();
                 let c_mins = c.mins();
-                let c_smap: HashMap<_, _> = HashMap::from_iter(c_mins.iter().zip(c_abunds.iter()));
+                let c_smap: HashMap<_, _> = c_mins.iter().zip(c_abunds.iter()).collect();
                 for item in a_smap.iter() {
                     assert!(c_smap.contains_key(*item.0));
                     assert!(c_smap.get(*item.0).unwrap() == item.1);
