@@ -15,7 +15,6 @@ use std::fmt::Debug;
 use std::fs::File;
 use std::hash::BuildHasherDefault;
 use std::io::{BufReader, Read};
-use std::iter::FromIterator;
 use std::path::{Path, PathBuf};
 use std::rc::Rc;
 
@@ -722,12 +721,7 @@ where
         let next_leaf = datasets.pop().unwrap();
 
         let (simleaf_tree, in_common) = if datasets.is_empty() {
-            (
-                BinaryTree::Empty,
-                HashSet::<u64, BuildHasherDefault<NoHashHasher<u64>>>::from_iter(
-                    next_leaf.mins().into_iter(),
-                ),
-            )
+            (BinaryTree::Empty, next_leaf.mins().into_iter().collect())
         } else {
             let mut similar_leaf_pos = 0;
             let mut current_max = 0;
@@ -741,16 +735,13 @@ where
 
             let similar_leaf = datasets.remove(similar_leaf_pos);
 
-            let in_common = HashSet::<u64, BuildHasherDefault<NoHashHasher<u64>>>::from_iter(
-                next_leaf.mins().into_iter(),
-            )
-            .union(
-                &HashSet::<u64, BuildHasherDefault<NoHashHasher<u64>>>::from_iter(
-                    similar_leaf.mins().into_iter(),
-                ),
-            )
-            .cloned()
-            .collect();
+            let in_common = next_leaf
+                .mins()
+                .into_iter()
+                .collect::<HashSet<u64, BuildHasherDefault<NoHashHasher<u64>>>>()
+                .union(&similar_leaf.mins().into_iter().collect())
+                .cloned()
+                .collect();
 
             let simleaf_tree = BinaryTree::Leaf(Box::new(TreeNode {
                 element: similar_leaf,
@@ -831,7 +822,7 @@ impl BinaryTree {
                 let mut similar_node_pos = 0;
                 let mut current_max = 0;
                 for (pos, cmpe) in current_round.iter().enumerate() {
-                    let common = BinaryTree::intersection_size(&next_node, &cmpe);
+                    let common = BinaryTree::intersection_size(&next_node, cmpe);
                     if common > current_max {
                         current_max = common;
                         similar_node_pos = pos;
