@@ -14,54 +14,119 @@ use rayon::prelude::*;
 use serde::{Deserialize, Serialize};
 use typed_builder::TypedBuilder;
 
-#[cfg(all(target_arch = "wasm32", target_vendor = "unknown"))]
-use wasm_bindgen::prelude::*;
-
+use crate::encodings::{aa_to_dayhoff, aa_to_hp, revcomp, to_aa, HashFunctions, VALID};
 use crate::index::storage::ToWriter;
-use crate::sketch::minhash::HashFunctions;
 use crate::sketch::Sketch;
 use crate::Error;
+use crate::HashIntoType;
 
 pub trait SigsTrait {
     fn size(&self) -> usize;
     fn to_vec(&self) -> Vec<u64>;
-    fn check_compatible(&self, other: &Self) -> Result<(), Error>;
-    fn add_sequence(&mut self, seq: &[u8], _force: bool) -> Result<(), Error>;
-    fn add_protein(&mut self, seq: &[u8]) -> Result<(), Error>;
     fn ksize(&self) -> usize;
+    fn check_compatible(&self, other: &Self) -> Result<(), Error>;
+    fn seed(&self) -> u64;
+
+    fn hash_function(&self) -> HashFunctions;
+
+    fn add_hash(&mut self, hash: HashIntoType);
+
+    fn add_sequence(&mut self, seq: &[u8], force: bool) -> Result<(), Error> {
+        let ready_hashes = SeqToHashes::new(
+            seq,
+            self.ksize(),
+            force,
+            false,
+            self.hash_function(),
+            self.seed(),
+        );
+
+        for hash_value in ready_hashes {
+            match hash_value {
+                Ok(0) => continue,
+                Ok(x) => self.add_hash(x),
+                Err(err) => return Err(err),
+            }
+        }
+
+        // Should be always ok
+        Ok(())
+    }
+
+    fn add_protein(&mut self, seq: &[u8]) -> Result<(), Error> {
+        let ready_hashes = SeqToHashes::new(
+            seq,
+            self.ksize(),
+            false,
+            true,
+            self.hash_function(),
+            self.seed(),
+        );
+
+        for hash_value in ready_hashes {
+            match hash_value {
+                Ok(0) => continue,
+                Ok(x) => self.add_hash(x),
+                Err(err) => return Err(err),
+            }
+        }
+
+        // Should be always ok
+        Ok(())
+    }
 }
 
 impl SigsTrait for Sketch {
     fn size(&self) -> usize {
         match *self {
-            Sketch::UKHS(ref ukhs) => ukhs.size(),
             Sketch::MinHash(ref mh) => mh.size(),
             Sketch::LargeMinHash(ref mh) => mh.size(),
+            Sketch::HyperLogLog(ref hll) => hll.size(),
         }
     }
 
     fn to_vec(&self) -> Vec<u64> {
         match *self {
-            Sketch::UKHS(ref ukhs) => ukhs.to_vec(),
             Sketch::MinHash(ref mh) => mh.to_vec(),
             Sketch::LargeMinHash(ref mh) => mh.to_vec(),
+            Sketch::HyperLogLog(ref hll) => hll.to_vec(),
         }
     }
 
     fn ksize(&self) -> usize {
         match *self {
-            Sketch::UKHS(ref ukhs) => ukhs.ksize(),
             Sketch::MinHash(ref mh) => mh.ksize(),
             Sketch::LargeMinHash(ref mh) => mh.ksize(),
+            Sketch::HyperLogLog(ref hll) => hll.ksize(),
+        }
+    }
+
+    fn seed(&self) -> u64 {
+        match *self {
+            Sketch::MinHash(ref mh) => mh.seed(),
+            Sketch::LargeMinHash(ref mh) => mh.seed(),
+            Sketch::HyperLogLog(ref hll) => hll.seed(),
+        }
+    }
+
+    fn hash_function(&self) -> HashFunctions {
+        match *self {
+            Sketch::MinHash(ref mh) => mh.hash_function(),
+            Sketch::LargeMinHash(ref mh) => mh.hash_function(),
+            Sketch::HyperLogLog(ref hll) => hll.hash_function(),
+        }
+    }
+
+    fn add_hash(&mut self, hash: HashIntoType) {
+        match *self {
+            Sketch::MinHash(ref mut mh) => mh.add_hash(hash),
+            Sketch::LargeMinHash(ref mut mh) => mh.add_hash(hash),
+            Sketch::HyperLogLog(ref mut hll) => hll.add_hash(hash),
         }
     }
 
     fn check_compatible(&self, other: &Self) -> Result<(), Error> {
         match *self {
-            Sketch::UKHS(ref ukhs) => match other {
-                Sketch::UKHS(ref ot) => ukhs.check_compatible(ot),
-                _ => Err(Error::MismatchSignatureType),
-            },
             Sketch::MinHash(ref mh) => match other {
                 Sketch::MinHash(ref ot) => mh.check_compatible(ot),
                 _ => Err(Error::MismatchSignatureType),
@@ -70,6 +135,10 @@ impl SigsTrait for Sketch {
                 Sketch::LargeMinHash(ref ot) => mh.check_compatible(ot),
                 _ => Err(Error::MismatchSignatureType),
             },
+            Sketch::HyperLogLog(ref hll) => match other {
+                Sketch::HyperLogLog(ref ot) => hll.check_compatible(ot),
+                _ => Err(Error::MismatchSignatureType),
+            },
         }
     }
 
@@ -77,7 +146,7 @@ impl SigsTrait for Sketch {
         match *self {
             Sketch::MinHash(ref mut mh) => mh.add_sequence(seq, force),
             Sketch::LargeMinHash(ref mut mh) => mh.add_sequence(seq, force),
-            Sketch::UKHS(_) => unimplemented!(),
+            Sketch::HyperLogLog(_) => unimplemented!(),
         }
     }
 
@@ -85,12 +154,233 @@ impl SigsTrait for Sketch {
         match *self {
             Sketch::MinHash(ref mut mh) => mh.add_protein(seq),
             Sketch::LargeMinHash(ref mut mh) => mh.add_protein(seq),
-            Sketch::UKHS(_) => unimplemented!(),
+            Sketch::HyperLogLog(_) => unimplemented!(),
+        }
+    }
+}
+
+// Iterator for converting sequence to hashes
+pub struct SeqToHashes {
+    sequence: Vec<u8>,
+    kmer_index: usize,
+    k_size: usize,
+    max_index: usize,
+    force: bool,
+    is_protein: bool,
+    hash_function: HashFunctions,
+    seed: u64,
+    hashes_buffer: Vec<u64>,
+
+    dna_configured: bool,
+    dna_rc: Vec<u8>,
+    dna_ksize: usize,
+    dna_len: usize,
+    dna_last_position_check: usize,
+
+    prot_configured: bool,
+    aa_seq: Vec<u8>,
+}
+
+impl SeqToHashes {
+    pub fn new(
+        seq: &[u8],
+        k_size: usize,
+        force: bool,
+        is_protein: bool,
+        hash_function: HashFunctions,
+        seed: u64,
+    ) -> SeqToHashes {
+        let mut ksize: usize = k_size;
+
+        // Divide the kmer size by 3 if protein
+        if is_protein || !hash_function.dna() {
+            ksize = k_size / 3;
+        }
+
+        // By setting _max_index to 0, the iterator will return None and exit
+        let _max_index: usize;
+        if seq.len() >= ksize {
+            _max_index = seq.len() - ksize + 1;
+        } else {
+            _max_index = 0;
+        }
+
+        SeqToHashes {
+            // Here we convert the sequence to upper case
+            sequence: seq.to_ascii_uppercase(),
+            k_size: ksize,
+            kmer_index: 0,
+            max_index: _max_index as usize,
+            force,
+            is_protein,
+            hash_function,
+            seed,
+            hashes_buffer: Vec::with_capacity(1000),
+            dna_configured: false,
+            dna_rc: Vec::with_capacity(1000),
+            dna_ksize: 0,
+            dna_len: 0,
+            dna_last_position_check: 0,
+            prot_configured: false,
+            aa_seq: Vec::new(),
+        }
+    }
+}
+
+impl Iterator for SeqToHashes {
+    type Item = Result<u64, Error>;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        // TODO: Remove the hashes buffer
+        // Priority for flushing the hashes buffer
+
+        if (self.kmer_index < self.max_index) || !self.hashes_buffer.is_empty() {
+            // Processing DNA or Translated DNA
+            if !self.is_protein {
+                // Setting the parameters only in the first iteration
+                if !self.dna_configured {
+                    self.dna_ksize = self.k_size as usize;
+                    self.dna_len = self.sequence.len();
+                    if self.dna_len < self.dna_ksize
+                        || (!self.hash_function.dna() && self.dna_len < self.k_size * 3)
+                    {
+                        return None;
+                    }
+                    // pre-calculate the reverse complement for the full sequence...
+                    self.dna_rc = revcomp(&self.sequence);
+                    self.dna_configured = true;
+                }
+
+                // Processing DNA
+                if self.hash_function.dna() {
+                    let kmer = &self.sequence[self.kmer_index..self.kmer_index + self.dna_ksize];
+
+                    for j in std::cmp::max(self.kmer_index, self.dna_last_position_check)
+                        ..self.kmer_index + self.dna_ksize
+                    {
+                        if !VALID[self.sequence[j] as usize] {
+                            if !self.force {
+                                return Some(Err(Error::InvalidDNA {
+                                    message: String::from_utf8(kmer.to_vec()).unwrap(),
+                                }));
+                            } else {
+                                self.kmer_index += 1;
+                                // Move the iterator to the next step
+                                return Some(Ok(0));
+                            }
+                        }
+                        self.dna_last_position_check += 1;
+                    }
+
+                    // ... and then while moving the k-mer window forward for the sequence
+                    // we move another window backwards for the RC.
+                    //   For a ksize = 3, and a sequence AGTCGT (len = 6):
+                    //                   +-+---------+---------------+-------+
+                    //   seq      RC     |i|i + ksize|len - ksize - i|len - i|
+                    //  AGTCGT   ACGACT  +-+---------+---------------+-------+
+                    //  +->         +->  |0|    2    |       3       |   6   |
+                    //   +->       +->   |1|    3    |       2       |   5   |
+                    //    +->     +->    |2|    4    |       1       |   4   |
+                    //     +->   +->     |3|    5    |       0       |   3   |
+                    //                   +-+---------+---------------+-------+
+                    // (leaving this table here because I had to draw to
+                    //  get the indices correctly)
+
+                    let krc = &self.dna_rc[self.dna_len - self.dna_ksize - self.kmer_index
+                        ..self.dna_len - self.kmer_index];
+                    let hash = crate::_hash_murmur(std::cmp::min(kmer, krc), self.seed);
+                    self.kmer_index += 1;
+                    Some(Ok(hash))
+                } else if self.hashes_buffer.is_empty() {
+                    // Processing protein by translating DNA
+                    // TODO: make it a real iterator not a buffer
+
+                    // Three frames
+                    for i in 0..3 {
+                        let substr: Vec<u8> = self
+                            .sequence
+                            .iter()
+                            .cloned()
+                            .skip(i)
+                            .take(self.sequence.len() - i)
+                            .collect();
+
+                        let aa = to_aa(
+                            &substr,
+                            self.hash_function.dayhoff(),
+                            self.hash_function.hp(),
+                        )
+                        .unwrap();
+
+                        aa.windows(self.k_size as usize).for_each(|n| {
+                            let hash = crate::_hash_murmur(n, self.seed);
+                            self.hashes_buffer.push(hash);
+                        });
+
+                        let rc_substr: Vec<u8> = self
+                            .dna_rc
+                            .iter()
+                            .cloned()
+                            .skip(i)
+                            .take(self.dna_rc.len() - i)
+                            .collect();
+                        let aa_rc = to_aa(
+                            &rc_substr,
+                            self.hash_function.dayhoff(),
+                            self.hash_function.hp(),
+                        )
+                        .unwrap();
+
+                        aa_rc.windows(self.k_size as usize).for_each(|n| {
+                            let hash = crate::_hash_murmur(n, self.seed);
+                            self.hashes_buffer.push(hash);
+                        });
+                    }
+                    self.kmer_index = self.max_index;
+                    Some(Ok(self.hashes_buffer.remove(0)))
+                } else {
+                    let first_element: u64 = self.hashes_buffer.remove(0);
+                    Some(Ok(first_element))
+                }
+            } else {
+                // Processing protein
+                // The kmer size is already divided by 3
+
+                if self.hash_function.protein() {
+                    let aa_kmer = &self.sequence[self.kmer_index..self.kmer_index + self.k_size];
+                    let hash = crate::_hash_murmur(aa_kmer, self.seed);
+                    self.kmer_index += 1;
+                    Some(Ok(hash))
+                } else {
+                    if !self.prot_configured {
+                        self.aa_seq = match self.hash_function {
+                            HashFunctions::murmur64_dayhoff => {
+                                self.sequence.iter().cloned().map(aa_to_dayhoff).collect()
+                            }
+                            HashFunctions::murmur64_hp => {
+                                self.sequence.iter().cloned().map(aa_to_hp).collect()
+                            }
+                            invalid => {
+                                return Some(Err(Error::InvalidHashFunction {
+                                    function: format!("{}", invalid),
+                                }));
+                            }
+                        };
+                    }
+
+                    let aa_kmer = &self.aa_seq[self.kmer_index..self.kmer_index + self.k_size];
+                    let hash = crate::_hash_murmur(aa_kmer, self.seed);
+                    self.kmer_index += 1;
+                    Some(Ok(hash))
+                }
+            }
+        } else {
+            // End the iterator
+            None
         }
     }
 }
 
-#[cfg_attr(all(target_arch = "wasm32", target_vendor = "unknown"), wasm_bindgen)]
 #[derive(Serialize, Deserialize, Debug, Clone, TypedBuilder)]
 pub struct Signature {
     #[serde(default = "default_class")]
@@ -197,7 +487,7 @@ impl Signature {
             match &self.signatures[0] {
                 Sketch::MinHash(mh) => mh.md5sum(),
                 Sketch::LargeMinHash(mh) => mh.md5sum(),
-                Sketch::UKHS(hs) => hs.md5sum(),
+                Sketch::HyperLogLog(_) => unimplemented!(),
             }
         } else {
             // TODO: select the correct signature
@@ -224,7 +514,7 @@ impl Signature {
 
     pub fn from_path<P: AsRef<Path>>(path: P) -> Result<Vec<Signature>, Error> {
         let mut reader = io::BufReader::new(File::open(path)?);
-        Ok(Signature::from_reader(&mut reader)?)
+        Signature::from_reader(&mut reader)
     }
 
     pub fn from_reader<R>(rdr: R) -> Result<Vec<Signature>, Error>
@@ -297,25 +587,7 @@ impl Signature {
                                 None => return true, // TODO: match previous behavior
                             };
                         }
-                        Sketch::UKHS(hs) => {
-                            if let Some(k) = ksize {
-                                if k != hs.ksize() as usize {
-                                    return false;
-                                }
-                            };
-
-                            match moltype {
-                                Some(x) => {
-                                    if x == HashFunctions::murmur64_DNA {
-                                        return true;
-                                    } else {
-                                        // TODO: draff only supports dna for now
-                                        unimplemented!()
-                                    }
-                                }
-                                None => unimplemented!(),
-                            };
-                        }
+                        Sketch::HyperLogLog(_) => unimplemented!(),
                     };
                     false
                 })
@@ -341,11 +613,9 @@ impl Signature {
                     sketch.add_sequence(&seq, force).unwrap(); }
                 );
         } else {
-            self.signatures
-                .iter_mut()
-                .for_each(|sketch| {
-                    sketch.add_sequence(&seq, force).unwrap(); }
-                );
+            for sketch in self.signatures.iter_mut(){
+                sketch.add_sequence(seq, force)?;
+            }
         }
         }
 
@@ -357,15 +627,15 @@ impl Signature {
         if #[cfg(feature = "parallel")] {
             self.signatures
                 .par_iter_mut()
-                .for_each(|sketch| {
-                    sketch.add_protein(&seq).unwrap(); }
-                );
+                .try_for_each(|sketch| {
+                    sketch.add_protein(&seq) }
+                )?;
         } else {
             self.signatures
                 .iter_mut()
-                .for_each(|sketch| {
-                    sketch.add_protein(&seq).unwrap(); }
-                );
+                .try_for_each(|sketch| {
+                    sketch.add_protein(seq) }
+                )?;
         }
         }
 
@@ -459,7 +729,7 @@ mod test {
         let reader = BufReader::new(file);
         let sigs: Vec<Signature> = serde_json::from_reader(reader).expect("Loading error");
 
-        assert_eq!(sigs.len(), 1);
+        assert_eq!(sigs.len(), 4);
 
         let sig = sigs.get(0).unwrap();
         assert_eq!(sig.class, "sourmash_signature");
@@ -469,9 +739,9 @@ mod test {
         }
         assert_eq!(sig.hash_function, "0.murmur64");
         if let Some(ref name) = sig.name {
-            assert_eq!(name, "s10+s11");
+            assert_eq!(name, "genome-s10+s11");
         }
-        assert_eq!(sig.signatures.len(), 4);
+        assert_eq!(sig.signatures.len(), 1);
     }
 
     #[test]
