@@ -5,7 +5,8 @@ objects.
 
 import pytest
 from sourmash import MinHash
-from . import sourmash_tst_utils as utils
+
+import sourmash_tst_utils as utils
 
 # below, 'track_abundance' is toggled to both True and False by py.test --
 # see conftest.py.
@@ -21,7 +22,7 @@ def test_jaccard_1(track_abundance):
         E2.add_hash(i)
 
     # here the union is [1, 2, 3, 4, 5]
-    # and the intesection is [1, 2, 3, 4] => 4/5.
+    # and the intersection is [1, 2, 3, 4] => 4/5.
 
     assert round(E1.jaccard(E2), 2) == round(4 / 5.0, 2)
     assert round(E2.jaccard(E1), 2) == round(4 / 5.0, 2)
@@ -76,18 +77,21 @@ def test_dna_mh(track_abundance):
     for i in range(len(seq) - 3):
         e2.add_kmer(seq[i:i + 4])
 
-    assert e1.get_mins() == e2.get_mins()
-    print(e1.get_mins())
-    assert 726311917625663847 in e1.get_mins()
-    assert 3697418565283905118 in e1.get_mins()
+    assert e1.hashes.keys() == e2.hashes.keys()
+    print(e1.hashes.keys())
+    assert 726311917625663847 in e1.hashes.keys()
+    assert 3697418565283905118 in e1.hashes.keys()
 
 
 def test_protein_mh(track_abundance):
-    e1 = MinHash(n=5, ksize=6, is_protein=True,
+    e1 = MinHash(n=5, ksize=2, is_protein=True,
                     track_abundance=track_abundance)
-    e2 = MinHash(n=5, ksize=6, is_protein=True,
+    e2 = MinHash(n=5, ksize=2, is_protein=True,
                     track_abundance=track_abundance)
 
+    # ok, so this is confusing, but: we are adding _DNA_ kmers here,
+    # and translating. so, add_sequence and add_kmer actually both add
+    # 6-mers.
     seq = 'ATGGCAGTGACGATGCCG'
     e1.add_sequence(seq)
 
@@ -95,8 +99,8 @@ def test_protein_mh(track_abundance):
         kmer = seq[i:i + 6]
         e2.add_kmer(kmer)
 
-    assert e1.get_mins() == e2.get_mins()
-    assert 901193879228338100 in e1.get_mins()
+    assert e1.hashes.keys() == e2.hashes.keys()
+    assert 901193879228338100 in e1.hashes.keys()
 
 
 def test_pickle(track_abundance):
@@ -116,12 +120,12 @@ def test_pickle(track_abundance):
     fp2 = BytesIO(fp.getvalue())
     e2 = pickle.load(fp2)
 
-    assert e1.get_mins(with_abundance=track_abundance) == \
-           e2.get_mins(with_abundance=track_abundance)
+    assert e1.hashes == e2.hashes
     assert e1.num == e2.num
     assert e1.ksize == e2.ksize
     assert e1.is_protein == e2.is_protein
-    assert e1.max_hash == e2.max_hash
+    assert e1.scaled == e2.scaled
+    assert e1.scaled == 0
     assert e1.seed == e2.seed
 
 
@@ -187,18 +191,18 @@ def test_jaccard_on_real_data():
     assert mh1.similarity(mh2) == 0.0183
     assert mh2.similarity(mh1) == 0.0183
 
-    mh1 = mh1.downsample_n(1000)
-    mh2 = mh2.downsample_n(1000)
+    mh1 = mh1.downsample(num=1000)
+    mh2 = mh2.downsample(num=1000)
     assert mh1.similarity(mh2) == 0.011
     assert mh2.similarity(mh1) == 0.011
 
-    mh1 = mh1.downsample_n(100)
-    mh2 = mh2.downsample_n(100)
+    mh1 = mh1.downsample(num=100)
+    mh2 = mh2.downsample(num=100)
     assert mh1.similarity(mh2) == 0.01
     assert mh2.similarity(mh1) == 0.01
 
-    mh1 = mh1.downsample_n(10)
-    mh2 = mh2.downsample_n(10)
+    mh1 = mh1.downsample(num=10)
+    mh2 = mh2.downsample(num=10)
     assert mh1.similarity(mh2) == 0.0
     assert mh2.similarity(mh1) == 0.0
 
@@ -219,27 +223,22 @@ def test_scaled_on_real_data():
     assert round(mh1.similarity(mh2), 5) == 0.01644
     assert round(mh2.similarity(mh1), 5) == 0.01644
 
-    mh1 = mh1.downsample_n(10000)
-    mh2 = mh2.downsample_n(10000)
+    mh1 = mh1.downsample(scaled=100)
+    mh2 = mh2.downsample(scaled=100)
+    assert round(mh1.similarity(mh2), 5) == 0.01644
+    assert round(mh2.similarity(mh1), 5) == 0.01644
 
-    assert mh1.similarity(mh2) == 0.0183
-    assert mh2.similarity(mh1) == 0.0183
+    mh1 = mh1.downsample(scaled=1000)
+    mh2 = mh2.downsample(scaled=1000)
+    assert round(mh1.similarity(mh2), 5) == 0.01874
+    assert round(mh2.similarity(mh1), 5) == 0.01874
 
-    mh1 = mh1.downsample_n(1000)
-    mh2 = mh2.downsample_n(1000)
-    assert mh1.similarity(mh2) == 0.011
-    assert mh2.similarity(mh1) == 0.011
+    mh1 = mh1.downsample(scaled=10000)
+    mh2 = mh2.downsample(scaled=10000)
 
-    mh1 = mh1.downsample_n(100)
-    mh2 = mh2.downsample_n(100)
     assert mh1.similarity(mh2) == 0.01
     assert mh2.similarity(mh1) == 0.01
 
-    mh1 = mh1.downsample_n(10)
-    mh2 = mh2.downsample_n(10)
-    assert mh1.similarity(mh2) == 0.0
-    assert mh2.similarity(mh1) == 0.0
-
 
 def test_scaled_on_real_data_2():
     from sourmash.signature import load_signatures
@@ -272,3 +271,17 @@ def test_scaled_on_real_data_2():
     mh2 = mh2.downsample(scaled=100000)
     assert round(mh1.similarity(mh2), 2) == 0.01
     assert round(mh2.similarity(mh1), 2) == 0.01
+
+
+def test_downsample_scaled_with_num():
+    from sourmash.signature import load_signatures
+
+    afile = 'scaled100/GCF_000005845.2_ASM584v2_genomic.fna.gz.sig.gz'
+    a = utils.get_test_data(afile)
+    sig1 = list(load_signatures(a))[0]
+    mh1 = sig1.minhash
+
+    with pytest.raises(ValueError) as exc:
+        mh = mh1.downsample(num=500)
+
+    assert 'cannot downsample a scaled MinHash using num' in str(exc.value)
