@@ -1,3 +1,4 @@
+"Test SBT code."
 import json
 import shutil
 import os
@@ -5,15 +6,17 @@ import os
 import pytest
 
 import sourmash
-from sourmash import load_one_signature, SourmashSignature, load_signatures
+from sourmash import (load_one_signature, SourmashSignature,
+                      load_file_as_signatures)
 from sourmash.exceptions import IndexNotSupported
 from sourmash.sbt import SBT, GraphFactory, Leaf, Node
-from sourmash.sbtmh import (SigLeaf, search_minhashes,
-                            search_minhashes_containment)
-from sourmash.sbt_storage import (FSStorage, TarStorage, RedisStorage,
+from sourmash.sbtmh import (SigLeaf, load_sbt_index)
+from sourmash.sbt_storage import (FSStorage, RedisStorage,
                                   IPFSStorage, ZipStorage)
+from sourmash.search import make_jaccard_search_query
+from sourmash.picklist import SignaturePicklist, PickStyle
 
-from . import sourmash_tst_utils as utils
+import sourmash_tst_utils as utils
 
 
 def test_simple(n_children):
@@ -51,12 +54,14 @@ def test_simple(n_children):
     root.add_node(leaf4)
     root.add_node(leaf5)
 
-    def search_kmer(obj, seq):
-        return obj.data.get(seq)
+    # return True if leaf node contains nodegraph w/kmer
+    def search_kmer(leaf, kmer):
+        return leaf.data.get(kmer)
 
     leaves = [leaf1, leaf2, leaf3, leaf4, leaf5 ]
     kmers = [ "AAAAA", "AAAAT", "AAAAG", "CAAAA", "GAAAA" ]
 
+    # define an exhaustive search function that looks in all the leaf nodes.
     def search_kmer_in_list(kmer):
         x = []
         for l in leaves:
@@ -65,22 +70,25 @@ def test_simple(n_children):
 
         return set(x)
 
+    # for all k-mers, ensure that tree._find_nodes matches the exhaustive
+    # search.
     for kmer in kmers:
-        assert set(root.find(search_kmer, kmer)) == search_kmer_in_list(kmer)
+        assert set(root._find_nodes(search_kmer, kmer)) == search_kmer_in_list(kmer)
 
     print('-----')
-    print([ x.metadata for x in root.find(search_kmer, "AAAAA") ])
-    print([ x.metadata for x in root.find(search_kmer, "AAAAT") ])
-    print([ x.metadata for x in root.find(search_kmer, "AAAAG") ])
-    print([ x.metadata for x in root.find(search_kmer, "CAAAA") ])
-    print([ x.metadata for x in root.find(search_kmer, "GAAAA") ])
+    print([ x.metadata for x in root._find_nodes(search_kmer, "AAAAA") ])
+    print([ x.metadata for x in root._find_nodes(search_kmer, "AAAAT") ])
+    print([ x.metadata for x in root._find_nodes(search_kmer, "AAAAG") ])
+    print([ x.metadata for x in root._find_nodes(search_kmer, "CAAAA") ])
+    print([ x.metadata for x in root._find_nodes(search_kmer, "GAAAA") ])
 
+    # save SBT to a directory and then reload
     with utils.TempDirectory() as location:
         root.save(os.path.join(location, 'demo'))
         root = SBT.load(os.path.join(location, 'demo'))
 
         for kmer in kmers:
-            new_result = {str(r) for r in root.find(search_kmer, kmer)}
+            new_result = {str(r) for r in root._find_nodes(search_kmer, kmer)}
             print(*new_result, sep='\n')
 
             assert new_result == {str(r) for r in search_kmer_in_list(kmer)}
@@ -132,19 +140,20 @@ def test_longer_search(n_children):
             return 1
         return 0
 
-    try1 = [ x.metadata for x in root.find(search_transcript, "AAAAT", 1.0) ]
+    try1 = [ x.metadata for x in root._find_nodes(search_transcript, "AAAAT", 1.0) ]
     assert set(try1) == set([ 'a', 'b', 'c', 'e' ]), try1 # no 'd'
 
-    try2 = [ x.metadata for x in root.find(search_transcript, "GAAAAAT", 0.6) ]
+    try2 = [ x.metadata for x in root._find_nodes(search_transcript, "GAAAAAT", 0.6) ]
     assert set(try2) == set([ 'a', 'b', 'c', 'd', 'e' ])
 
-    try3 = [ x.metadata for x in root.find(search_transcript, "GAAAA", 1.0) ]
+    try3 = [ x.metadata for x in root._find_nodes(search_transcript, "GAAAA", 1.0) ]
     assert set(try3) == set([ 'd', 'e' ]), try3
 
 
-@pytest.mark.parametrize("old_version", ["v1", "v2", "v3", "v4", "v5"])
+#@pytest.mark.parametrize("old_version", ["v1", "v2", "v3", "v4", "v5"])
+@pytest.mark.parametrize("old_version", ["v3", "v4", "v5"])
 def test_tree_old_load(old_version):
-    tree_v1 = SBT.load(utils.get_test_data('{}.sbt.json'.format(old_version)),
+    tree_old = SBT.load(utils.get_test_data('{}.sbt.json'.format(old_version)),
                        leaf_loader=SigLeaf.load)
 
     tree_cur = SBT.load(utils.get_test_data('v6.sbt.json'),
@@ -153,13 +162,19 @@ def test_tree_old_load(old_version):
     testdata1 = utils.get_test_data(utils.SIG_FILES[0])
     to_search = load_one_signature(testdata1)
 
-    results_v1 = {str(s) for s in tree_v1.find(search_minhashes_containment,
-                                               to_search, 0.1)}
-    results_cur = {str(s) for s in tree_cur.find(search_minhashes_containment,
-                                                 to_search, 0.1)}
+    print(list(tree_old.leaves()))
 
-    assert results_v1 == results_cur
-    assert len(results_v1) == 4
+    # note: earlier versions of this test did containment on
+    # the num MinHash in `to_search`, which doesn't work properly.
+    # (See test_sbt_no_containment_on_num for test). So, to
+    # fix the test for the new search API, we had to adjust
+    # the threshold.
+    search_obj = make_jaccard_search_query(threshold=0.05)
+    results_old = {str(s.signature) for s in tree_old.find(search_obj, to_search)}
+    results_cur = {str(s.signature) for s in tree_cur.find(search_obj, to_search)}
+
+    assert results_old == results_cur
+    assert len(results_old) == 4
 
 
 def test_load_future(tmpdir):
@@ -184,8 +199,8 @@ def test_tree_save_load(n_children):
 
     print('*' * 60)
     print("{}:".format(to_search.metadata))
-    old_result = {str(s) for s in tree.find(search_minhashes,
-                                            to_search.data, 0.1)}
+    search_obj = make_jaccard_search_query(threshold=0.1)
+    old_result = {str(s.signature) for s in tree.find(search_obj, to_search.data)}
     print(*old_result, sep='\n')
 
     with utils.TempDirectory() as location:
@@ -195,8 +210,8 @@ def test_tree_save_load(n_children):
 
         print('*' * 60)
         print("{}:".format(to_search.metadata))
-        new_result = {str(s) for s in tree.find(search_minhashes,
-                                                to_search.data, 0.1)}
+        search_obj = make_jaccard_search_query(threshold=0.1)
+        new_result = {str(s.signature) for s in tree.find(search_obj, to_search.data)}
         print(*new_result, sep='\n')
 
         assert old_result == new_result
@@ -214,12 +229,15 @@ def test_search_minhashes():
 
     to_search = next(iter(tree.leaves()))
 
-    # this fails if 'search_minhashes' is calc containment and not similarity.
-    results = tree.find(search_minhashes, to_search.data, 0.08)
-    for leaf in results:
-        assert to_search.data.similarity(leaf.data) >= 0.08
+    # this fails if 'search_obj' is calc containment and not similarity.
+    search_obj = make_jaccard_search_query(threshold=0.08)
+    results = tree.find(search_obj, to_search.data)
 
-    print(results)
+    n = 0
+    for n, sr in enumerate(results):
+        assert to_search.data.jaccard(sr.signature) >= 0.08
+
+    assert n == 1
 
 
 def test_binary_nary_tree():
@@ -244,7 +262,8 @@ def test_binary_nary_tree():
     print('*' * 60)
     print("{}:".format(to_search.metadata))
     for d, tree in trees.items():
-        results[d] = {str(s) for s in tree.find(search_minhashes, to_search.data, 0.1)}
+        search_obj = make_jaccard_search_query(threshold=0.1)
+        results[d] = {str(s.signature) for s in tree.find(search_obj, to_search.data)}
     print(*results[2], sep='\n')
 
     assert results[2] == results[5]
@@ -278,10 +297,9 @@ def test_sbt_combine(n_children):
     assert t1_leaves == t_leaves
 
     to_search = load_one_signature(utils.get_test_data(utils.SIG_FILES[0]))
-    t1_result = {str(s) for s in tree_1.find(search_minhashes,
-                                             to_search, 0.1)}
-    tree_result = {str(s) for s in tree.find(search_minhashes,
-                                             to_search, 0.1)}
+    search_obj = make_jaccard_search_query(threshold=0.1)
+    t1_result = {str(s.signature) for s in tree_1.find(search_obj, to_search)}
+    tree_result = {str(s.signature) for s in tree.find(search_obj, to_search)}
     assert t1_result == tree_result
 
     # TODO: save and load both trees
@@ -295,7 +313,7 @@ def test_sbt_combine(n_children):
     if not next_empty:
         next_empty = n + 1
 
-    tree_1.add_node(SigLeaf(to_search.name(), to_search))
+    tree_1.add_node(SigLeaf(to_search.name, to_search))
     assert tree_1.next_node == next_empty
 
 
@@ -313,18 +331,18 @@ def test_sbt_fsstorage():
 
         print('*' * 60)
         print("{}:".format(to_search.metadata))
-        old_result = {str(s) for s in tree.find(search_minhashes,
-                                                to_search.data, 0.1)}
+        search_obj = make_jaccard_search_query(threshold=0.1)
+        old_result = {str(s.signature) for s in tree.find(search_obj, to_search.data)}
         print(*old_result, sep='\n')
 
         with FSStorage(location, '.fstree') as storage:
-            tree.save(os.path.join(location, 'tree'), storage=storage)
+            tree.save(os.path.join(location, 'tree.sbt.json'), storage=storage)
 
-        tree = SBT.load(os.path.join(location, 'tree'), leaf_loader=SigLeaf.load)
+        tree = SBT.load(os.path.join(location, 'tree.sbt.json'), leaf_loader=SigLeaf.load)
         print('*' * 60)
         print("{}:".format(to_search.metadata))
-        new_result = {str(s) for s in tree.find(search_minhashes,
-                                                to_search.data, 0.1)}
+        search_obj = make_jaccard_search_query(threshold=0.1)
+        new_result = {str(s.signature) for s in tree.find(search_obj, to_search.data)}
         print(*new_result, sep='\n')
 
         assert old_result == new_result
@@ -333,41 +351,6 @@ def test_sbt_fsstorage():
         assert os.path.exists(os.path.join(location, '.fstree'))
 
 
-def test_sbt_tarstorage():
-    factory = GraphFactory(31, 1e5, 4)
-    with utils.TempDirectory() as location:
-        tree = SBT(factory)
-
-        for f in utils.SIG_FILES:
-            sig = load_one_signature(utils.get_test_data(f))
-
-            leaf = SigLeaf(os.path.basename(f), sig)
-            tree.add_node(leaf)
-            to_search = leaf
-
-        print('*' * 60)
-        print("{}:".format(to_search.metadata))
-        old_result = {str(s) for s in tree.find(search_minhashes,
-                                                to_search.data, 0.1)}
-        print(*old_result, sep='\n')
-
-        with TarStorage(os.path.join(location, 'tree.tar.gz')) as storage:
-            tree.save(os.path.join(location, 'tree'), storage=storage)
-
-        with TarStorage(os.path.join(location, 'tree.tar.gz')) as storage:
-            tree = SBT.load(os.path.join(location, 'tree'),
-                            leaf_loader=SigLeaf.load,
-                            storage=storage)
-
-            print('*' * 60)
-            print("{}:".format(to_search.metadata))
-            new_result = {str(s) for s in tree.find(search_minhashes,
-                                                    to_search.data, 0.1)}
-            print(*new_result, sep='\n')
-
-            assert old_result == new_result
-
-
 def test_sbt_zipstorage(tmpdir):
     # create tree, save to a zip, then load and search.
     factory = GraphFactory(31, 1e5, 4)
@@ -375,29 +358,29 @@ def test_sbt_zipstorage(tmpdir):
     tree = SBT(factory)
 
     for f in utils.SIG_FILES:
-        sig = next(load_signatures(utils.get_test_data(f)))
+        sig = next(load_file_as_signatures(utils.get_test_data(f)))
         leaf = SigLeaf(os.path.basename(f), sig)
         tree.add_node(leaf)
         to_search = leaf
 
     print('*' * 60)
     print("{}:".format(to_search.metadata))
-    old_result = {str(s) for s in tree.find(search_minhashes,
-                                            to_search.data, 0.1)}
+    search_obj = make_jaccard_search_query(threshold=0.1)
+    old_result = {str(s.signature) for s in tree.find(search_obj, to_search.data)}
     print(*old_result, sep='\n')
 
     with ZipStorage(str(tmpdir.join("tree.sbt.zip"))) as storage:
-        tree.save(str(tmpdir.join("tree")), storage=storage)
+        tree.save(str(tmpdir.join("tree.sbt.json")), storage=storage)
 
     with ZipStorage(str(tmpdir.join("tree.sbt.zip"))) as storage:
-        tree = SBT.load(str(tmpdir.join("tree")),
+        tree = SBT.load(str(tmpdir.join("tree.sbt.json")),
                         leaf_loader=SigLeaf.load,
                         storage=storage)
 
         print('*' * 60)
         print("{}:".format(to_search.metadata))
-        new_result = {str(s) for s in tree.find(search_minhashes,
-                                                to_search.data, 0.1)}
+        search_obj = make_jaccard_search_query(threshold=0.1)
+        new_result = {str(s.signature) for s in tree.find(search_obj, to_search.data)}
         print(*new_result, sep='\n')
 
         assert old_result == new_result
@@ -419,25 +402,25 @@ def test_sbt_ipfsstorage():
 
         print('*' * 60)
         print("{}:".format(to_search.metadata))
-        old_result = {str(s) for s in tree.find(search_minhashes,
-                                                to_search.data, 0.1)}
+        search_obj = make_jaccard_search_query(threshold=0.1)
+        old_result = {str(s.signature) for s in tree.find(search_obj, to_search.data)}
         print(*old_result, sep='\n')
 
         try:
             with IPFSStorage() as storage:
-                tree.save(os.path.join(location, 'tree'), storage=storage)
+                tree.save(os.path.join(location, 'tree.sbt.json'), storage=storage)
         except ipfshttpclient.exceptions.ConnectionError:
             pytest.xfail("ipfs not installed/functioning probably")
 
         with IPFSStorage() as storage:
-            tree = SBT.load(os.path.join(location, 'tree'),
+            tree = SBT.load(os.path.join(location, 'tree.sbt.json'),
                             leaf_loader=SigLeaf.load,
                             storage=storage)
 
             print('*' * 60)
             print("{}:".format(to_search.metadata))
-            new_result = {str(s) for s in tree.find(search_minhashes,
-                                                    to_search.data, 0.1)}
+            search_obj = make_jaccard_search_query(threshold=0.1)
+            new_result = {str(s.signature) for s in tree.find(search_obj, to_search.data)}
             print(*new_result, sep='\n')
 
             assert old_result == new_result
@@ -458,25 +441,25 @@ def test_sbt_redisstorage():
 
         print('*' * 60)
         print("{}:".format(to_search.metadata))
-        old_result = {str(s) for s in tree.find(search_minhashes,
-                                                to_search.data, 0.1)}
+        search_obj = make_jaccard_search_query(threshold=0.1)
+        old_result = {str(s.signature) for s in tree.find(search_obj, to_search.data)}
         print(*old_result, sep='\n')
 
         try:
             with RedisStorage() as storage:
-                tree.save(os.path.join(location, 'tree'), storage=storage)
+                tree.save(os.path.join(location, 'tree.sbt.json'), storage=storage)
         except redis.exceptions.ConnectionError:
             pytest.xfail("Couldn't connect to redis server")
 
         with RedisStorage() as storage:
-            tree = SBT.load(os.path.join(location, 'tree'),
+            tree = SBT.load(os.path.join(location, 'tree.sbt.json'),
                             leaf_loader=SigLeaf.load,
                             storage=storage)
 
             print('*' * 60)
             print("{}:".format(to_search.metadata))
-            new_result = {str(s) for s in tree.find(search_minhashes,
-                                                    to_search.data, 0.1)}
+            search_obj = make_jaccard_search_query(threshold=0.1)
+            new_result = {str(s.signature) for s in tree.find(search_obj, to_search.data)}
             print(*new_result, sep='\n')
 
             assert old_result == new_result
@@ -502,8 +485,9 @@ def test_save_zip(tmpdir):
 
     print("*" * 60)
     print("{}:".format(to_search))
-    old_result = {str(s) for s in tree.find(search_minhashes, to_search, 0.1)}
-    new_result = {str(s) for s in new_tree.find(search_minhashes, to_search, 0.1)}
+    search_obj = make_jaccard_search_query(threshold=0.1)
+    old_result = {str(s.signature) for s in tree.find(search_obj, to_search)}
+    new_result = {str(s.signature) for s in new_tree.find(search_obj, to_search)}
     print(*new_result, sep="\n")
 
     assert old_result == new_result
@@ -523,7 +507,8 @@ def test_load_zip(tmpdir):
 
     print("*" * 60)
     print("{}:".format(to_search))
-    new_result = {str(s) for s in tree.find(search_minhashes, to_search, 0.1)}
+    search_obj = make_jaccard_search_query(threshold=0.1)
+    new_result = {str(s.signature) for s in tree.find(search_obj, to_search)}
     print(*new_result, sep="\n")
     assert len(new_result) == 2
 
@@ -544,7 +529,8 @@ def test_load_zip_uncompressed(tmpdir):
 
     print("*" * 60)
     print("{}:".format(to_search))
-    new_result = {str(s) for s in tree.find(search_minhashes, to_search, 0.1)}
+    search_obj = make_jaccard_search_query(threshold=0.1)
+    new_result = {str(s.signature) for s in tree.find(search_obj, to_search)}
     print(*new_result, sep="\n")
     assert len(new_result) == 2
 
@@ -559,10 +545,9 @@ def test_tree_repair():
     testdata1 = utils.get_test_data(utils.SIG_FILES[0])
     to_search = load_one_signature(testdata1)
 
-    results_repair = {str(s) for s in tree_repair.find(search_minhashes,
-                                                       to_search, 0.1)}
-    results_cur = {str(s) for s in tree_cur.find(search_minhashes,
-                                                 to_search, 0.1)}
+    search_obj = make_jaccard_search_query(threshold=0.1)
+    results_repair = {str(s.signature) for s in tree_repair.find(search_obj, to_search)}
+    results_cur = {str(s.signature) for s in tree_cur.find(search_obj, to_search)}
 
     assert results_repair == results_cur
     assert len(results_repair) == 2
@@ -600,8 +585,9 @@ def test_save_sparseness(n_children):
 
     print('*' * 60)
     print("{}:".format(to_search.metadata))
-    old_result = {str(s) for s in tree.find(search_minhashes,
-                                            to_search.data, 0.1)}
+
+    search_obj = make_jaccard_search_query(threshold=0.1)
+    old_result = {str(s.signature) for s in tree.find(search_obj, to_search.data)}
     print(*old_result, sep='\n')
 
     with utils.TempDirectory() as location:
@@ -612,8 +598,8 @@ def test_save_sparseness(n_children):
 
         print('*' * 60)
         print("{}:".format(to_search.metadata))
-        new_result = {str(s) for s in tree_loaded.find(search_minhashes,
-                                                       to_search.data, 0.1)}
+        new_result = {str(s.signature) for s in tree_loaded.find(search_obj,
+                                                       to_search.data)}
         print(*new_result, sep='\n')
 
         assert old_result == new_result
@@ -646,12 +632,168 @@ def test_sbt_as_index_select():
     xx = tree.select(moltype='DNA')
     assert xx == tree
 
+    xx = tree.select(abund=False)
+    assert xx == tree
+
     with pytest.raises(ValueError):
         tree.select(ksize=21)
 
     with pytest.raises(ValueError):
         tree.select(moltype='protein')
 
+    with pytest.raises(ValueError):
+        tree.select(abund=True)
+
+
+def test_sbt_as_index_select_picklist():
+    # test 'select' method from Index base class with a picklist
+
+    factory = GraphFactory(31, 1e5, 4)
+    tree = SBT(factory, d=2)
+
+    sig47 = load_one_signature(utils.get_test_data('47.fa.sig'))
+    sig63 = load_one_signature(utils.get_test_data('63.fa.sig'))
+
+    tree.insert(sig47)
+    tree.insert(sig63)
+
+    # construct a picklist...
+    picklist = SignaturePicklist('md5prefix8')
+    picklist.init(['09a08691'])
+
+    # select on picklist
+    tree = tree.select(picklist=picklist)
+    siglist = list(tree.signatures())
+    assert len(siglist) == 1
+
+    ss = siglist[0]
+    assert ss.minhash.ksize == 31
+    assert ss.md5sum().startswith('09a08691c')
+
+
+def test_sbt_as_index_select_picklist_exclude():
+    # test 'select' method from Index base class with a picklist, exclude
+
+    factory = GraphFactory(31, 1e5, 4)
+    tree = SBT(factory, d=2)
+
+    sig47 = load_one_signature(utils.get_test_data('47.fa.sig'))
+    sig63 = load_one_signature(utils.get_test_data('63.fa.sig'))
+
+    tree.insert(sig47)
+    tree.insert(sig63)
+
+    # construct a picklist...
+    picklist = SignaturePicklist('md5prefix8', pickstyle=PickStyle.EXCLUDE)
+    picklist.init(['09a08691'])
+
+    # select on picklist
+    tree = tree.select(picklist=picklist)
+    siglist = list(tree.signatures())
+    assert len(siglist) == 1
+
+    ss = siglist[0]
+    assert ss.minhash.ksize == 31
+    assert ss.md5sum().startswith('38729c637')
+
+
+def test_sbt_as_index_find_picklist():
+    # test 'select' method from Index base class with a picklist
+
+    factory = GraphFactory(31, 1e5, 4)
+    tree = SBT(factory, d=2)
+
+    sig47 = load_one_signature(utils.get_test_data('47.fa.sig'))
+    sig63 = load_one_signature(utils.get_test_data('63.fa.sig'))
+
+    tree.insert(sig47)
+    tree.insert(sig63)
+
+    # construct a picklist...
+    picklist = SignaturePicklist('md5prefix8')
+    picklist.init(['09a08691'])
+
+    # run a 'find' with sig63, should find 47 and 63 both.
+    search_obj = make_jaccard_search_query(do_containment=True, threshold=0.0)
+    results = list(tree.find(search_obj, sig63))
+    print(results)
+    assert len(results) == 2
+
+    # now, select on picklist and do another find...
+    tree = tree.select(picklist=picklist)
+    results = list(tree.find(search_obj, sig63))
+    print(results)
+    assert len(results) == 1
+
+    # and check that it is the expected one!
+    ss = results[0].signature
+    assert ss.minhash.ksize == 31
+    assert ss.md5sum().startswith('09a08691c')
+
+
+def test_sbt_as_index_find_picklist_exclude():
+    # test 'select' method from Index base class with a picklist
+
+    factory = GraphFactory(31, 1e5, 4)
+    tree = SBT(factory, d=2)
+
+    sig47 = load_one_signature(utils.get_test_data('47.fa.sig'))
+    sig63 = load_one_signature(utils.get_test_data('63.fa.sig'))
+
+    tree.insert(sig47)
+    tree.insert(sig63)
+
+    # construct a picklist...
+    picklist = SignaturePicklist('md5prefix8', pickstyle=PickStyle.EXCLUDE)
+    picklist.init(['09a08691'])
+
+    # run a 'find' with sig63, should find 47 and 63 both.
+    search_obj = make_jaccard_search_query(do_containment=True, threshold=0.0)
+    results = list(tree.find(search_obj, sig63))
+    print(results)
+    assert len(results) == 2
+
+    # now, select on picklist and do another find...
+    tree = tree.select(picklist=picklist)
+    results = list(tree.find(search_obj, sig63))
+    print(results)
+    assert len(results) == 1
+
+    # and check that it is the expected one!
+    ss = results[0].signature
+    assert ss.minhash.ksize == 31
+    assert ss.md5sum().startswith('38729c637')
+
+
+def test_sbt_as_index_find_picklist_twice():
+    # test 'select' method from Index base class with a picklist
+
+    factory = GraphFactory(31, 1e5, 4)
+    tree = SBT(factory, d=2)
+
+    sig47 = load_one_signature(utils.get_test_data('47.fa.sig'))
+    sig63 = load_one_signature(utils.get_test_data('63.fa.sig'))
+
+    tree.insert(sig47)
+    tree.insert(sig63)
+
+    # construct a picklist...
+    picklist = SignaturePicklist('md5prefix8')
+    picklist.init(['09a08691'])
+
+    # run a 'find' with sig63, should find 47 and 63 both.
+    search_obj = make_jaccard_search_query(do_containment=True, threshold=0.0)
+    results = list(tree.find(search_obj, sig63))
+    print(results)
+    assert len(results) == 2
+
+    # now, select twice on picklists...
+    tree = tree.select(picklist=picklist)
+
+    with pytest.raises(ValueError):
+        tree = tree.select(picklist=picklist)
+        assert "we do not (yet) support multiple picklists for SBT databases" in str(exc)
+
 
 def test_sbt_as_index_signatures():
     # test 'signatures' method from Index base class.
@@ -687,12 +829,13 @@ def test_sbt_gather_threshold_1():
     # now construct query signatures with specific numbers of hashes --
     # note, these signatures all have scaled=1000.
 
-    mins = list(sorted(sig2.minhash.get_mins()))
+    mins = list(sorted(sig2.minhash.hashes.keys()))
     new_mh = sig2.minhash.copy_and_clear()
 
     # query with empty hashes
     assert not new_mh
-    assert not tree.gather(SourmashSignature(new_mh))
+    with pytest.raises(ValueError):
+        tree.gather(SourmashSignature(new_mh))
 
     # add one hash
     new_mh.add_hash(mins.pop())
@@ -706,8 +849,8 @@ def test_sbt_gather_threshold_1():
     assert name is None
 
     # check with a threshold -> should be no results.
-    results = tree.gather(SourmashSignature(new_mh), threshold_bp=5000)
-    assert not results
+    with pytest.raises(ValueError):
+        tree.gather(SourmashSignature(new_mh), threshold_bp=5000)
 
     # add three more hashes => length of 4
     new_mh.add_hash(mins.pop())
@@ -724,8 +867,8 @@ def test_sbt_gather_threshold_1():
 
     # check with a too-high threshold -> should be no results.
     print('len mh', len(new_mh))
-    results = tree.gather(SourmashSignature(new_mh), threshold_bp=5000)
-    assert not results
+    with pytest.raises(ValueError):
+        tree.gather(SourmashSignature(new_mh), threshold_bp=5000)
 
 
 def test_sbt_gather_threshold_5():
@@ -744,7 +887,7 @@ def test_sbt_gather_threshold_5():
     # now construct query signatures with specific numbers of hashes --
     # note, these signatures all have scaled=1000.
 
-    mins = list(sorted(sig2.minhash.get_mins()))
+    mins = list(sorted(sig2.minhash.hashes.keys()))
     new_mh = sig2.minhash.copy_and_clear()
 
     # add five hashes
@@ -773,8 +916,8 @@ def test_sbt_gather_threshold_5():
 
 
 @utils.in_tempdir
-def test_gather_multiple_return(c):
-    # test gather() method number of returns
+def test_gather_single_return(c):
+    # test gather() number of returns
     sig2file = utils.get_test_data('2.fa.sig')
     sig47file = utils.get_test_data('47.fa.sig')
     sig63file = utils.get_test_data('63.fa.sig')
@@ -795,12 +938,13 @@ def test_gather_multiple_return(c):
     # right order?
     results = tree.gather(sig63)
     print(len(results))
-    assert len(results) == 2
+    assert len(results) == 1
     assert results[0][0] == 1.0
 
 
-@utils.in_tempdir
-def test_sbt_protein_command_index(c):
+def test_sbt_protein_command_index(runtmp):
+    c = runtmp
+
     # test command-line creation of SBT database with protein sigs
     sigfile1 = utils.get_test_data('prot/protein/GCA_001593925.1_ASM159392v1_protein.faa.gz.sig')
     sigfile2 = utils.get_test_data('prot/protein/GCA_001593935.1_ASM159393v1_protein.faa.gz.sig')
@@ -808,9 +952,12 @@ def test_sbt_protein_command_index(c):
     db_out = c.output('protein.sbt.zip')
 
     c.run_sourmash('index', db_out, sigfile1, sigfile2,
-                   '--scaled', '100', '-k', '57', '--protein')
+                   '--scaled', '100', '-k', '19', '--protein')
 
-    db2 = sourmash.load_sbt_index(db_out)
+    # check to make sure .sbt.protein directory doesn't get created
+    assert not os.path.exists(c.output('.sbt.protein'))
+
+    db2 = load_sbt_index(db_out)
 
     sig1 = sourmash.load_one_signature(sigfile1)
     sig2 = sourmash.load_one_signature(sigfile2)
@@ -828,12 +975,34 @@ def test_sbt_protein_command_index(c):
 
     results = db2.gather(sig2)
     assert results[0][0] == 1.0
+    assert results[0][2] == db2._location
+    assert results[0][2] == db_out
+
+
+@utils.in_tempdir
+def test_sbt_protein_search_no_threshold(c):
+    # test the '.search' method on SBTs w/no threshold
+    sigfile1 = utils.get_test_data('prot/protein/GCA_001593925.1_ASM159392v1_protein.faa.gz.sig')
+    sigfile2 = utils.get_test_data('prot/protein/GCA_001593935.1_ASM159393v1_protein.faa.gz.sig')
+
+    db_out = c.output('protein.sbt.zip')
+
+    c.run_sourmash('index', db_out, sigfile1, sigfile2,
+                   '--scaled', '100', '-k', '19', '--protein')
+
+    db2 = load_sbt_index(db_out)
+
+    sig1 = sourmash.load_one_signature(sigfile1)
+
+    # and search, gather
+    with pytest.raises(TypeError) as exc:
+        results = db2.search(sig1)
+    assert "'search' requires 'threshold'" in str(exc)
 
 
 @utils.in_thisdir
 def test_sbt_protein_command_search(c):
-    # test command-line search/gather of LCA database with protein sigs
-    # (LCA database created as above)
+    # test command-line search/gather of SBT database with protein sigs
     sigfile1 = utils.get_test_data('prot/protein/GCA_001593925.1_ASM159392v1_protein.faa.gz.sig')
     db_out = utils.get_test_data('prot/protein.sbt.zip')
 
@@ -854,9 +1023,9 @@ def test_sbt_hp_command_index(c):
     db_out = c.output('hp.sbt.zip')
 
     c.run_sourmash('index', db_out, sigfile1, sigfile2,
-                   '--scaled', '100', '-k', '57', '--hp')
+                   '--scaled', '100', '-k', '19', '--hp')
 
-    db2 = sourmash.load_sbt_index(db_out)
+    db2 = load_sbt_index(db_out)
 
     sig1 = sourmash.load_one_signature(sigfile1)
     sig2 = sourmash.load_one_signature(sigfile2)
@@ -874,12 +1043,13 @@ def test_sbt_hp_command_index(c):
 
     results = db2.gather(sig2)
     assert results[0][0] == 1.0
+    assert results[0][2] == db2._location
+    assert results[0][2] == db_out
 
 
 @utils.in_thisdir
 def test_sbt_hp_command_search(c):
-    # test command-line search/gather of LCA database with hp sigs
-    # (LCA database created as above)
+    # test command-line search/gather of SBT database with hp sigs
     sigfile1 = utils.get_test_data('prot/hp/GCA_001593925.1_ASM159392v1_protein.faa.gz.sig')
     db_out = utils.get_test_data('prot/hp.sbt.zip')
 
@@ -900,9 +1070,9 @@ def test_sbt_dayhoff_command_index(c):
     db_out = c.output('dayhoff.sbt.zip')
 
     c.run_sourmash('index', db_out, sigfile1, sigfile2,
-                   '--scaled', '100', '-k', '57', '--dayhoff')
+                   '--scaled', '100', '-k', '19', '--dayhoff')
 
-    db2 = sourmash.load_sbt_index(db_out)
+    db2 = load_sbt_index(db_out)
 
     sig1 = sourmash.load_one_signature(sigfile1)
     sig2 = sourmash.load_one_signature(sigfile2)
@@ -920,12 +1090,13 @@ def test_sbt_dayhoff_command_index(c):
 
     results = db2.gather(sig2)
     assert results[0][0] == 1.0
+    assert results[0][2] == db2._location
+    assert results[0][2] == db_out
 
 
 @utils.in_thisdir
 def test_sbt_dayhoff_command_search(c):
-    # test command-line search/gather of LCA database with dayhoff sigs
-    # (LCA database created as above)
+    # test command-line search/gather of SBT database with dayhoff sigs
     sigfile1 = utils.get_test_data('prot/dayhoff/GCA_001593925.1_ASM159392v1_protein.faa.gz.sig')
     db_out = utils.get_test_data('prot/dayhoff.sbt.zip')
 
@@ -935,3 +1106,107 @@ def test_sbt_dayhoff_command_search(c):
     c.run_sourmash('gather', sigfile1, db_out, '--threshold', '0.0')
     assert 'found 1 matches total' in c.last_result.out
     assert 'the recovered matches hit 100.0% of the query' in c.last_result.out
+
+
+def test_sbt_node_cache():
+    tree = SBT.load(utils.get_test_data('v6.sbt.json'),
+                    leaf_loader=SigLeaf.load,
+                    cache_size=1)
+
+    testdata1 = utils.get_test_data(utils.SIG_FILES[0])
+    to_search = load_one_signature(testdata1)
+
+    # note: earlier versions of this test did containment on
+    # the num MinHash in `to_search`, which doesn't work properly.
+    # (See test_sbt_no_containment_on_num for test). So, to
+    # fix the test for the new search API, we had to adjust
+    # the threshold.
+    search_obj = make_jaccard_search_query(threshold=0.05)
+    results = list(tree.find(search_obj, to_search))
+    assert len(results) == 4
+
+    assert tree._nodescache.currsize == 1
+    assert tree._nodescache.currsize == 1
+
+
+def test_sbt_no_containment_on_num():
+    tree = SBT.load(utils.get_test_data('v6.sbt.json'),
+                    leaf_loader=SigLeaf.load,
+                    cache_size=1)
+
+    testdata1 = utils.get_test_data(utils.SIG_FILES[0])
+    to_search = load_one_signature(testdata1)
+
+    search_obj = make_jaccard_search_query(do_containment=True, threshold=0.05)
+    with pytest.raises(TypeError) as exc:
+        results = list(tree.find(search_obj, to_search))
+
+    assert "this search requires a scaled signature" in str(exc)
+
+
+def test_build_sbt_zip_with_dups(runtmp):
+    dups_data = utils.get_test_data('duplicate-sigs')
+
+    all_sigs = set(sourmash.load_file_as_signatures(dups_data))
+    assert len(all_sigs) == 4
+
+    runtmp.run_sourmash('index', 'dups.sbt.zip', dups_data)
+    outfile = runtmp.output('dups.sbt.zip')
+
+    sbt_sigs = set(sourmash.load_file_as_signatures(outfile))
+    assert len(sbt_sigs) == 4
+
+    assert all_sigs == sbt_sigs
+
+
+def test_build_sbt_zip_with_dups_exists(runtmp):
+    dups_data = utils.get_test_data('duplicate-sigs')
+
+    all_sigs = set(sourmash.load_file_as_signatures(dups_data))
+    assert len(all_sigs) == 4
+
+    runtmp.run_sourmash('index', 'dups.sbt.zip', dups_data)
+    outfile = runtmp.output('dups.sbt.zip')
+
+    # run again, to see what happens :)
+    runtmp.run_sourmash('index', 'dups.sbt.zip', dups_data)
+    outfile = runtmp.output('dups.sbt.zip')
+
+    sbt_sigs = set(sourmash.load_file_as_signatures(outfile))
+    assert len(sbt_sigs) == 4
+
+    assert all_sigs == sbt_sigs
+
+
+def test_build_sbt_json_with_dups(runtmp):
+    dups_data = utils.get_test_data('duplicate-sigs')
+
+    all_sigs = set(sourmash.load_file_as_signatures(dups_data))
+    assert len(all_sigs) == 4
+
+    runtmp.run_sourmash('index', 'dups.sbt.json', dups_data)
+    outfile = runtmp.output('dups.sbt.json')
+
+    sbt_sigs = set(sourmash.load_file_as_signatures(outfile))
+    assert len(sbt_sigs) == 4
+
+    assert all_sigs == sbt_sigs
+
+
+def test_build_sbt_json_with_dups_exists(runtmp):
+    dups_data = utils.get_test_data('duplicate-sigs')
+
+    all_sigs = set(sourmash.load_file_as_signatures(dups_data))
+    assert len(all_sigs) == 4
+
+    runtmp.run_sourmash('index', 'dups.sbt.json', dups_data)
+    outfile = runtmp.output('dups.sbt.json')
+
+    # run again, see what happens!
+    runtmp.run_sourmash('index', 'dups.sbt.json', dups_data)
+    outfile = runtmp.output('dups.sbt.json')
+
+    sbt_sigs = set(sourmash.load_file_as_signatures(outfile))
+    assert len(sbt_sigs) == 4
+
+    assert all_sigs == sbt_sigs
