@@ -10,7 +10,8 @@ import json
 import csv
 import pytest
 
-from . import sourmash_tst_utils as utils
+import sourmash_tst_utils as utils
+
 import sourmash
 from sourmash import MinHash
 from sourmash.sbt import SBT, Node
@@ -21,6 +22,7 @@ from sourmash.cli import SourmashParser
 
 from sourmash import signature
 from sourmash import VERSION
+from sourmash_tst_utils import SourmashCommandFailed
 
 
 def test_do_sourmash_compute():
@@ -34,7 +36,44 @@ def test_do_sourmash_compute():
         assert os.path.exists(sigfile)
 
         sig = next(signature.load_signatures(sigfile))
-        assert sig.name().endswith('short.fa')
+        assert str(sig).endswith('short.fa')
+
+
+def test_do_sourmash_compute_check_num_bounds_negative(runtmp):
+    c=runtmp
+    testdata1 = utils.get_test_data('short.fa')
+    testdata2 = utils.get_test_data('short2.fa')
+    testdata3 = utils.get_test_data('short3.fa')
+    sigfile = c.output('short.fa.sig')
+
+    with pytest.raises(SourmashCommandFailed):
+        c.run_sourmash('compute', '-k', '31', '--num-hashes', '-5', '-o', sigfile, '--merge', '"name"', testdata1, testdata2, testdata3)
+    
+    assert "ERROR: --num-hashes value must be positive" in c.last_result.err
+
+
+def test_do_sourmash_compute_check_num_bounds_less_than_minimum(runtmp):
+    c=runtmp
+    testdata1 = utils.get_test_data('short.fa')
+    testdata2 = utils.get_test_data('short2.fa')
+    testdata3 = utils.get_test_data('short3.fa')
+    sigfile = c.output('short.fa.sig')
+
+    c.run_sourmash('compute', '-k', '31', '--num-hashes', '25', '-o', sigfile, '--merge', '"name"', testdata1, testdata2, testdata3)
+    
+    assert "WARNING: --num-hashes value should be >= 50. Continuing anyway." in c.last_result.err
+
+
+def test_do_sourmash_compute_check_num_bounds_more_than_maximum(runtmp):
+    c=runtmp
+    testdata1 = utils.get_test_data('short.fa')
+    testdata2 = utils.get_test_data('short2.fa')
+    testdata3 = utils.get_test_data('short3.fa')
+    sigfile = c.output('short.fa.sig')
+
+    c.run_sourmash('compute', '-k', '31', '--num-hashes', '100000', '-o', sigfile, '--merge', '"name"', testdata1, testdata2, testdata3)
+    
+    assert "WARNING: --num-hashes value should be <= 50000. Continuing anyway." in c.last_result.err
 
 
 @utils.in_tempdir
@@ -49,7 +88,7 @@ def test_do_sourmash_compute_outdir(c):
     assert os.path.exists(sigfile)
 
     sig = next(signature.load_signatures(sigfile))
-    assert sig.name().endswith('short.fa')
+    assert str(sig).endswith('short.fa')
 
 
 def test_do_sourmash_compute_output_valid_file():
@@ -108,7 +147,7 @@ def test_do_sourmash_compute_output_and_name_valid_file(c):
     c.run_sourmash('compute', '-k', '31', '-o', sigfile, '--merge', '"name"', testdata1, testdata2, testdata3)
 
     assert os.path.exists(sigfile)
-    assert 'calculated 1 signatures for 4 sequences taken from 3 files' in c.last_result.err
+    assert 'calculated 1 signature for 4 sequences taken from 3 files' in c.last_result.err
 
     # is it valid json?
     with open(sigfile, 'r') as f:
@@ -132,7 +171,7 @@ def test_do_sourmash_compute_output_and_name_valid_file_outdir(c):
     testdata3 = utils.get_test_data('short3.fa')
     sigfile = os.path.join(c.location, 'short.fa.sig')
 
-    with pytest.raises(ValueError) as exc:
+    with pytest.raises(SourmashCommandFailed) as exc:
         c.run_sourmash('compute', '-k', '31', '-o', sigfile,
                        '--merge', '"name"',
                        testdata1, testdata2, testdata3,
@@ -154,136 +193,7 @@ def test_do_sourmash_compute_singleton():
         assert os.path.exists(sigfile)
 
         sig = next(signature.load_signatures(sigfile))
-        assert sig.name().endswith('shortName')
-
-
-def test_do_sourmash_compute_10x_barcode():
-    pytest.importorskip('bam2fasta')
-
-    with utils.TempDirectory() as location:
-        testdata1 = utils.get_test_data('10x-example/possorted_genome_bam.bam')
-        barcodes_file = utils.get_test_data('10x-example/barcodes.tsv')
-        status, out, err = utils.runscript('sourmash',
-                                           ['compute', '-k', '21',
-                                            '--line-count', '50',
-                                            '--input-is-10x',
-                                            '--protein',
-                                            '--barcodes-file',
-                                            barcodes_file,
-                                            testdata1],
-                                           in_directory=location)
-
-        sigfile = os.path.join(location, 'possorted_genome_bam.bam.sig')
-        assert os.path.exists(sigfile)
-        siglist = list(signature.load_signatures(sigfile))
-        assert len(siglist) == 16
-        barcode_signatures = list(set([sig.name().split("_")[0] for sig in siglist]))
-
-        with open(utils.get_test_data('10x-example/barcodes.tsv')) as f:
-            true_barcodes = set(x.strip() for x in f.readlines())
-
-        # Ensure that every cell barcode in barcodes.tsv has a signature
-        assert all(bc in true_barcodes for bc in barcode_signatures)
-        # TODO PV This seems to randomly fail/pass - commenting out for now
-        # but the min hashes should never be empty
-        # min_hashes = [x.minhash.get_mins() for x in siglist]
-        # assert all(mins != [] for mins in min_hashes)
-
-
-def test_do_sourmash_compute_10x_no_barcode():
-    pytest.importorskip('bam2fasta')
-    # Filtered bam file with no barcodes file
-    # should run sourmash compute successfully
-    with utils.TempDirectory() as location:
-        testdata1 = utils.get_test_data('10x-example/possorted_genome_bam_filtered.bam')
-        status, out, err = utils.runscript('sourmash',
-                                           ['compute', '-k', '31',
-                                            '--dna',
-                                            '--input-is-10x',
-                                            testdata1,
-                                            '-o', '10x-example_dna.sig'],
-                                           in_directory=location)
-
-        sigfile = os.path.join(location, '10x-example_dna.sig')
-        assert os.path.exists(sigfile)
-        siglist = list(signature.load_signatures(sigfile))
-        assert len(siglist) == 32
-        # TODO PV This seems to randomly fail/pass - commenting out for now
-        # but the min hashes should never be empty
-        # min_hashes = [x.minhash.get_mins() for x in siglist]
-        # assert all(mins != [] for mins in min_hashes)
-
-
-def test_do_sourmash_compute_10x_no_filter_umis():
-    pytest.importorskip('bam2fasta')
-    with utils.TempDirectory() as location:
-        # test to check if all the lines in unfiltered_umi_to_sig are callled and tested
-        csv_path = os.path.join(location, "all_barcodes_meta.csv")
-        testdata1 = utils.get_test_data('10x-example/possorted_genome_bam_filtered.bam')
-        status, out, err = utils.runscript('sourmash',
-                                           ['compute', '-k', '31',
-                                            '--dna',
-                                            '--input-is-10x',
-                                            testdata1,
-                                            '--write-barcode-meta-csv', csv_path,
-                                            '--save-fastas', location,
-                                            '-o', '10x-example_dna.sig'],
-                                           in_directory=location)
-        sigfile = os.path.join(location, '10x-example_dna.sig')
-        assert os.path.exists(sigfile)
-        siglist = list(signature.load_signatures(sigfile))
-        assert len(siglist) == 32
-
-
-def test_do_sourmash_compute_10x_filter_umis():
-    pytest.importorskip('bam2fasta')
-    with utils.TempDirectory() as location:
-        testdata1 = utils.get_test_data('10x-example/possorted_genome_bam.bam')
-        csv_path = os.path.join(location, "all_barcodes_meta.csv")
-        barcodes_path = utils.get_test_data('10x-example/barcodes.tsv')
-        renamer_path = utils.get_test_data('10x-example/barcodes_renamer.tsv')
-        fastas_dir = os.path.join(location, "fastas")
-        if not os.path.exists(fastas_dir):
-            os.makedirs(fastas_dir)
-
-        status, out, err = utils.runscript('sourmash',
-                                           ['compute', '-k', '31',
-                                            '--dna', '--count-valid-reads', '10',
-                                            '--input-is-10x',
-                                            testdata1,
-                                            '--write-barcode-meta-csv', csv_path,
-                                            '--barcodes', barcodes_path,
-                                            '--rename-10x-barcodes', renamer_path,
-                                            '--save-fastas', fastas_dir,
-                                            '-o', '10x-example_dna.sig'],
-                                           in_directory=location)
-
-        sigfile = os.path.join(location, '10x-example_dna.sig')
-        assert os.path.exists(sigfile)
-        siglist = list(signature.load_signatures(sigfile))
-        assert len(siglist) == 1
-        # TODO PV This seems to randomly fail/pass - commenting out for now
-        # but the min hashes should never be empty
-        # min_hashes = [x.minhash.get_mins() for x in siglist]
-        # assert all(mins != [] for mins in min_hashes)
-
-        with open(csv_path, 'rb') as f:
-            data = [line.split() for line in f]
-        assert len(data) == 9
-        fasta_files = os.listdir(fastas_dir)
-        barcodes = [filename.replace(".fasta", "") for filename in fasta_files]
-        assert len(barcodes) == 1
-        assert len(fasta_files) == 1
-        assert barcodes[0] == 'lung_epithelial_cell|AAATGCCCAAACTGCT-1'
-        count = 0
-        fasta_file_name = os.path.join(fastas_dir, fasta_files[0])
-        for record in screed.open(fasta_file_name):
-            name = record.name
-            sequence = record.sequence
-            count += 1
-            assert name.startswith('lung_epithelial_cell|AAATGCCCAAACTGCT-1')
-            assert sequence.count(">") == 0
-            assert sequence.count("X") == 0
+        assert sig.name.endswith('shortName')
 
 
 def test_do_sourmash_compute_name():
@@ -298,7 +208,7 @@ def test_do_sourmash_compute_name():
         assert os.path.exists(sigfile)
 
         sig = next(signature.load_signatures(sigfile))
-        assert sig.name() == 'foo'
+        assert sig.name == 'foo'
 
         status, out, err = utils.runscript('sourmash',
                                            ['compute', '-k', '31', '--name', 'foo',
@@ -309,8 +219,8 @@ def test_do_sourmash_compute_name():
         assert os.path.exists(sigfile2)
 
         sig2 = next(signature.load_signatures(sigfile))
-        assert sig2.name() == 'foo'
-        assert sig.name() == sig2.name()
+        assert sig2.name == 'foo'
+        assert sig.name == sig2.name
 
 
 def test_do_sourmash_compute_name_fail_no_output():
@@ -354,7 +264,7 @@ def test_do_sourmash_compute_name_from_first():
         assert os.path.exists(sigfile)
 
         sig = next(signature.load_signatures(sigfile))
-        assert sig.name() == 'firstname'
+        assert sig.name == 'firstname'
 
 
 def test_do_sourmash_compute_multik():
@@ -372,6 +282,7 @@ def test_do_sourmash_compute_multik():
         ksizes = set([ x.minhash.ksize for x in siglist ])
         assert 21 in ksizes
         assert 31 in ksizes
+        assert len(ksizes) == 2
 
 
 def test_do_sourmash_compute_multik_with_protein():
@@ -392,6 +303,9 @@ def test_do_sourmash_compute_multik_with_protein():
             ksizes = set([ x.minhash.ksize for x in siglist ])
             assert 21 in ksizes
             assert 30 in ksizes
+            assert 7 in ksizes
+            assert 10 in ksizes
+            assert len(ksizes) == 4
 
 
 def test_do_sourmash_compute_multik_with_dayhoff():
@@ -412,9 +326,10 @@ def test_do_sourmash_compute_multik_with_dayhoff():
             siglist = list(signature.load_signatures(sigdata))
             assert len(siglist) == 2
             ksizes = set([ x.minhash.ksize for x in siglist ])
-            assert 21 in ksizes
-            assert 30 in ksizes
+            assert 7 in ksizes
+            assert 10 in ksizes
             assert all(x.minhash.dayhoff for x in siglist)
+            assert len(ksizes) == 2
 
 
 def test_do_sourmash_compute_multik_with_dayhoff_and_dna():
@@ -435,8 +350,11 @@ def test_do_sourmash_compute_multik_with_dayhoff_and_dna():
             ksizes = set([ x.minhash.ksize for x in siglist ])
             assert 21 in ksizes
             assert 30 in ksizes
+            assert 7 in ksizes
+            assert 10 in ksizes
             assert sum(x.minhash.moltype == 'DNA' for x in siglist) == 2
             assert sum(x.minhash.moltype == 'dayhoff' for x in siglist) == 2
+            assert len(ksizes) == 4
 
 
 def test_do_sourmash_compute_multik_with_hp():
@@ -457,9 +375,10 @@ def test_do_sourmash_compute_multik_with_hp():
             siglist = list(signature.load_signatures(sigdata))
             assert len(siglist) == 2
             ksizes = set([ x.minhash.ksize for x in siglist ])
-            assert 21 in ksizes
-            assert 30 in ksizes
+            assert 7 in ksizes
+            assert 10 in ksizes
             assert all(x.minhash.hp for x in siglist)
+            assert len(ksizes) == 2
 
 
 def test_do_sourmash_compute_multik_with_hp_and_dna():
@@ -473,6 +392,17 @@ def test_do_sourmash_compute_multik_with_hp_and_dna():
         outfile = os.path.join(location, 'short.fa.sig')
         assert os.path.exists(outfile)
 
+        with open(outfile, 'rt') as fp:
+            sigdata = fp.read()
+            siglist = list(signature.load_signatures(sigdata))
+            assert len(siglist) == 4
+            ksizes = set([ x.minhash.ksize for x in siglist ])
+            assert 7 in ksizes
+            assert 10 in ksizes
+            assert 21 in ksizes
+            assert 30 in ksizes
+            assert len(ksizes) == 4
+
 
 def test_do_sourmash_compute_multik_with_dayhoff_dna_protein():
     with utils.TempDirectory() as location:
@@ -492,9 +422,12 @@ def test_do_sourmash_compute_multik_with_dayhoff_dna_protein():
             ksizes = set([ x.minhash.ksize for x in siglist ])
             assert 21 in ksizes
             assert 30 in ksizes
+            assert 7 in ksizes
+            assert 10 in ksizes
             assert sum(x.minhash.moltype == 'DNA' for x in siglist) == 2
             assert sum(x.minhash.moltype == 'dayhoff' for x in siglist) == 2
             assert sum(x.minhash.moltype == 'protein' for x in siglist) == 2
+            assert len(ksizes) == 4
 
 
 def test_do_sourmash_compute_multik_with_dayhoff_hp_dna_protein():
@@ -513,6 +446,8 @@ def test_do_sourmash_compute_multik_with_dayhoff_hp_dna_protein():
             siglist = list(signature.load_signatures(sigdata))
             assert len(siglist) == 8
             ksizes = set([ x.minhash.ksize for x in siglist ])
+            assert 7 in ksizes
+            assert 10 in ksizes
             assert 21 in ksizes
             assert 30 in ksizes
             assert sum(x.minhash.moltype == 'DNA' for x in siglist) == 2
@@ -520,6 +455,7 @@ def test_do_sourmash_compute_multik_with_dayhoff_hp_dna_protein():
             assert sum(x.minhash.moltype == 'hp' for x in siglist) == 2
             # 2 = dayhoff, 2 = hp = 4 protein
             assert sum(x.minhash.moltype == 'protein' for x in siglist) == 2
+            assert len(ksizes) == 4
 
 
 def test_do_sourmash_compute_multik_with_nothing():
@@ -563,11 +499,12 @@ def test_do_sourmash_compute_multik_only_protein(c):
         siglist = list(signature.load_signatures(sigdata))
         assert len(siglist) == 2
         ksizes = set([ x.minhash.ksize for x in siglist ])
-        assert 21 in ksizes
-        assert 30 in ksizes
+        assert 7 in ksizes
+        assert 10 in ksizes
+        assert len(ksizes) == 2
 
 
-def test_do_sourmash_compute_multik_protein_input_non_div3_ksize():
+def test_do_sourmash_compute_multik_protein_input_bad_ksize():
     with utils.TempDirectory() as location:
         testdata1 = utils.get_test_data('short-protein.fa')
         status, out, err = utils.runscript('sourmash',
@@ -578,7 +515,8 @@ def test_do_sourmash_compute_multik_protein_input_non_div3_ksize():
                                            in_directory=location,
                                            fail_ok=True)
         outfile = os.path.join(location, 'short-protein.fa.sig')
-        assert os.path.exists(outfile)
+        assert status != 0
+        assert 'protein ksizes must be divisible by 3' in err
 
 
 @utils.in_tempdir
@@ -596,8 +534,9 @@ def test_do_sourmash_compute_multik_only_protein_no_rna(c):
         siglist = list(signature.load_signatures(sigdata))
         assert len(siglist) == 2
         ksizes = set([ x.minhash.ksize for x in siglist ])
-        assert 21 in ksizes
-        assert 30 in ksizes
+        assert 7 in ksizes
+        assert 10 in ksizes
+        assert len(ksizes) == 2
 
 
 def test_do_sourmash_compute_protein_bad_sequences():
@@ -617,8 +556,9 @@ def test_do_sourmash_compute_protein_bad_sequences():
             siglist = list(signature.load_signatures(sigdata))
             assert len(siglist) == 2
             ksizes = set([ x.minhash.ksize for x in siglist ])
-            assert 21 in ksizes
-            assert 30 in ksizes
+            assert 7 in ksizes
+            assert 10 in ksizes
+            assert len(ksizes) == 2
 
 
 def test_do_sourmash_compute_multik_input_is_protein():
@@ -637,8 +577,9 @@ def test_do_sourmash_compute_multik_input_is_protein():
             siglist = list(signature.load_signatures(sigdata))
             assert len(siglist) == 2
             ksizes = set([ x.minhash.ksize for x in siglist ])
-            assert 21 in ksizes
-            assert 30 in ksizes
+            assert 7 in ksizes
+            assert 10 in ksizes
+            assert len(ksizes) == 2
 
             moltype = set([ x.minhash.moltype == 'protein'
                             for x in siglist ])
@@ -677,9 +618,9 @@ def test_do_sourmash_compute_with_scaled_1():
         siglist = list(signature.load_signatures(outfile))
         assert len(siglist) == 2
 
-        max_hashes = [ x.minhash.max_hash for x in siglist ]
-        assert len(max_hashes) == 2
-        assert set(max_hashes) == { sourmash.MAX_HASH }
+        scaled_vals = [ x.minhash.scaled for x in siglist ]
+        assert len(scaled_vals) == 2
+        assert set(scaled_vals) == { 1 }
 
 
 def test_do_sourmash_compute_with_scaled_2():
@@ -696,7 +637,7 @@ def test_do_sourmash_compute_with_scaled_2():
         siglist = list(signature.load_signatures(outfile))
         assert len(siglist) == 2
 
-        max_hashes = [ x.minhash.max_hash for x in siglist ]
+        max_hashes = [ x.minhash._max_hash for x in siglist ]
         assert len(max_hashes) == 2
         assert set(max_hashes) == set([ int(2**64 /2.) ])
 
@@ -715,7 +656,7 @@ def test_do_sourmash_compute_with_scaled():
         siglist = list(signature.load_signatures(outfile))
         assert len(siglist) == 2
 
-        max_hashes = [ x.minhash.max_hash for x in siglist ]
+        max_hashes = [ x.minhash._max_hash for x in siglist ]
         assert len(max_hashes) == 2
         assert set(max_hashes) == set([ int(2**64 /100.) ])
 
@@ -798,18 +739,18 @@ def test_do_sourmash_check_protein_comparisons():
 
         # I'm not sure why load_signatures is randomizing order, but ok.
         x = list(signature.load_signatures(sig1))
-        sig1_aa, sig2_aa = sorted(x, key=lambda x: x.name())
+        sig1_aa, sig2_aa = sorted(x, key=lambda x: x.name)
 
         x = list(signature.load_signatures(sig2))
-        sig1_trans, sig2_trans = sorted(x, key=lambda x: x.name())
+        sig1_trans, sig2_trans = sorted(x, key=lambda x: x.name)
 
-        name1 = sig1_aa.name().split()[0]
+        name1 = sig1_aa.name.split()[0]
         assert name1 == 'NP_414543.1'
-        name2 = sig2_aa.name().split()[0]
+        name2 = sig2_aa.name.split()[0]
         assert name2 == 'NP_414544.1'
-        name3 = sig1_trans.name().split()[0]
+        name3 = sig1_trans.name.split()[0]
         assert name3 == 'gi|556503834:2801-3733'
-        name4 = sig2_trans.name().split()[0]
+        name4 = sig2_trans.name.split()[0]
         assert name4 == 'gi|556503834:337-2799'
 
         print(name1, name3, round(sig1_aa.similarity(sig1_trans), 3))
@@ -835,7 +776,9 @@ def test_do_sourmash_check_knowngood_dna_comparisons(c):
     assert os.path.exists(sig1)
 
     x = list(signature.load_signatures(sig1))
-    sig1, sig2 = sorted(x, key=lambda x: x.name())
+    sig1, sig2 = sorted(x, key=lambda x: x.name)
+    print(sig1.name)
+    print(sig2.name)
 
     knowngood = utils.get_test_data('benchmark.dna.sig')
     good = list(signature.load_signatures(knowngood))[0]
@@ -853,7 +796,7 @@ def test_do_sourmash_check_knowngood_dna_comparisons_use_rna(c):
     assert os.path.exists(sig1)
 
     x = list(signature.load_signatures(sig1))
-    sig1, sig2 = sorted(x, key=lambda x: x.name())
+    sig1, sig2 = sorted(x, key=lambda x: x.name)
 
     knowngood = utils.get_test_data('benchmark.dna.sig')
     good = list(signature.load_signatures(knowngood))[0]
@@ -876,7 +819,7 @@ def test_do_sourmash_check_knowngood_input_protein_comparisons():
         assert os.path.exists(sig1)
 
         x = list(signature.load_signatures(sig1))
-        sig1_aa, sig2_aa = sorted(x, key=lambda x: x.name())
+        sig1_aa, sig2_aa = sorted(x, key=lambda x: x.name)
 
         knowngood = utils.get_test_data('benchmark.input_prot.sig')
         good_aa = list(signature.load_signatures(knowngood))[0]
@@ -899,7 +842,7 @@ def test_do_sourmash_check_knowngood_protein_comparisons():
         assert os.path.exists(sig1)
 
         x = list(signature.load_signatures(sig1))
-        sig1_trans, sig2_trans = sorted(x, key=lambda x: x.name())
+        sig1_trans, sig2_trans = sorted(x, key=lambda x: x.name)
 
         knowngood = utils.get_test_data('benchmark.prot.sig')
         good_trans = list(signature.load_signatures(knowngood))[0]
